from EzCheck import *
from DataModel import *
from EzCustomerMovement import *
from EzRFM import *
from EzCohort import *

check = EzCheck()
dm = DataModel()
rfm = EzRFM()
cm = EzCustomerMovement()
ch = EzCohort()
libs = [check,dm,rfm,cm,ch]
if __name__ == "__main__":
    for lib in libs:
        print(f"{lib.prep_data}")


import pandas as pd
import numpy as np
from datetime import datetime
import time

class EzCheck:
    def __init__(self):
        self.prep_data='EzCheck'

    def get_null(self, x):
        res = x.isna().sum()
        return res

    def get_dtypes(self, x):
        res = x.dtypes
        return res

    def get_nunique(self, x):
        res = x.nunique()
        return res

    def get_outlier(self, data):
        columns = data.columns.values
        res = []
        for col in columns:
            if data[col].dtypes.name in ['int8','int16','int32','int64',
                                           'float16','float32','float64']:
                q1,q3 = data[col].quantile([.25,.75])
                IQR = (q3-q1)*1.5
                lower_bound = q1-IQR
                upper_bound = q3+IQR

                if (data[col] < lower_bound).sum() != 0 | (data[col] > upper_bound).sum() != 0:
                    res.append('yes')
                else:
                    res.append('no')
            else:
                res.append('undefined')
        return res

    def convert_dtypes(self, data,column):
        pass

    def check(self, data, n=4):
        check_df = pd.DataFrame()
        data = data
        n_row, n_col = data.shape
        pipeline = {'dtypes':self.get_dtypes,'missing':self.get_null,
                    'missing_ratio':self.get_null,
                    'nunique':self.get_nunique,
                    'n_ratio':self.get_nunique}
        for col, func in pipeline.items():
            if 'ratio' not in col:
                check_df[col] = func(data)
            else:
                check_df[col] = func(data)/n_row

        check_df['outlier'] = self.get_outlier(data)

        return check_df.round(n)

    def get_num(self,x):
        arr = pd.Series([x.count(),min(x),x.quantile(.25),x.median(),x.mean(),
                         x.quantile(.75),max(x),max(x)-min(x),x.skew(),x.std()], 
                        index=['count','min','25%','median','mean','75','max','range','skew','std'])
        return arr

    def get_obj(self,x):
        arr = pd.Series([x.count(),x.value_counts().index[0],x.value_counts().values[0],
                         x.value_counts().index[-1],x.value_counts().values[-1]],
                        index=['count','top','t_freq','least','l_freq'])
        return arr

    def get_date(self,x):
        arr = pd.Series([x.count(),x.min(),x.max(), x.max()-x.min()],
                        index=['count','min','max','day'])
        return arr

    def summary(self, data, method):
        data = data.select_dtypes(method)
        columns = data.columns
        summary_df = pd.DataFrame()
        for col in columns:

            if method=='number':
                summary_df[col] = self.get_num(data[col])
                
            elif method=='object':
                summary_df[col] = self.get_obj(data[col])

            elif method=='datetime':
                summary_df[col] = self.get_date(data[col])

        return summary_df.T.round(4)

    def sumNum(self, data, method='number'):
        return self.summary(data, method)

    def sumCat(self, data, method='object'):
        return self.summary(data, method)

    def sumDate(self, data, method='datetime'):
        return self.summary(data, method)


import pandas as pd
import numpy as np
from datetime import datetime
import time
from .DataModel import DataModel

class EzRFM(DataModel):
    def __init__(self):
        super().__init__()
        self.prep_data = 'EzRMF'

    def prep_rfmt(self, data):
        target = data[self.orderDate].max() + np.timedelta64(1,'D')
        data = data.groupby(self.customerId).agg(
            totalRecency=pd.NamedAgg(column=self.orderDate, aggfunc=lambda x: (target-x.max()).days),
            totalFrequency=pd.NamedAgg(column=self.orderId, aggfunc='nunique'),
            totalMonetary=pd.NamedAgg(column=self.salesPrice, aggfunc='sum'),
            totalTenure=pd.NamedAgg(column=self.orderDate, aggfunc=lambda x: (target-x.min()).days),
            totalSizeMin=pd.NamedAgg(column=self.salesPrice, aggfunc='min'),
            totalSizeQ1=pd.NamedAgg(column=self.salesPrice, aggfunc=lambda x: x.quantile(.25)),
            totalSizeMean=pd.NamedAgg(column=self.salesPrice, aggfunc='mean'),
            totalSizeMedian=pd.NamedAgg(column=self.salesPrice, aggfunc='median'),
            totalSizeQ3=pd.NamedAgg(column=self.salesPrice, aggfunc=lambda x: x.quantile(.75)),
            totalSizeMax=pd.NamedAgg(column=self.salesPrice, aggfunc='max'),
            totalSizeStd=pd.NamedAgg(column=self.salesPrice, aggfunc='std'),
            totalSizeRange=pd.NamedAgg(column=self.salesPrice, aggfunc=lambda x: x.max()-x.min()),
            totalSizeSkew=pd.NamedAgg(column=self.salesPrice, aggfunc='skew'),)
        
        return data.reset_index().fillna(0).round(2)

    def prep_rfmt_date(self, data, column, method):
        data = data.groupby([self.customerId,column]).agg({
            self.orderId:'nunique',
            self.salesPrice:'sum'
        }).reset_index()

        data = data.groupby(self.customerId).agg({
            column:'nunique',
            self.orderId:['mean','median','std'],
            self.salesPrice:['min', self.get_q1, 'mean', 'median', 
                             self.get_q3, 'max','std', self.get_range,'skew']
        }).reset_index()

        names = ['FrequencySum','FrequencyMean','FrequencyMedian','FrequencyStd',
                 'MonetaryMin','MonetaryQ1','MonetaryMean','MonetaryMedian',
                 'MonetaryQ3','MonetaryMax','MonetaryStd','MonetaryRange','MonetarySkew']
        data.columns = [self.customerId] + [method +  name for name in names]

        return data

    def fit(self, data, customerId, orderId, orderDate, salesPrice, method=None):
        
        start = time.time()

        init_df = self.prep(data, customerId, orderId, orderDate, salesPrice)
        rfmt_df = self.prep_rfmt(init_df)
        # total_visit = rfmt_df['totalFrequency']

        method_list = ['year','quarter','month','week'] 

        if method == None:
            rfmt_df = rfmt_df
        else:
            if method == 'year':
                columns = ['year']
            elif method == 'quarter':
                columns = ['yearQuarter']
            elif method == 'month':
                columns = ['yearMonth']
            elif method == 'week':
                columns = ['yearWeek']
            elif method == 'full':
                columns = ['year','yearQuarter','yearMonth','yearWeek']

            for enum, column in enumerate(columns):        
                rfmt_date = self.prep_rfmt_date(init_df, column, method_list[enum])
                rfmt_df = pd.concat([rfmt_df, rfmt_date.drop('customerId',axis=1)],axis=1)

        print('successfully preped EzRFM {:.4f} ms'.format(time.time()-start))

        return rfmt_df


from distutils.core import setup

# try:
#     import pypandoc
#     long_description = pypandoc.convert('README.md', 'rst')
# except(IOError, ImportError):
#     long_description = open('README.md').read()

setup(
  name = 'ggwp',
  packages = ['ggwp'],
  version = '0.0.8',
  license='MIT',
  description = 'Prepare Fast, Analyze Faster',
  # long_description = long_description,
  author = 'pathompol',
  author_email = 'data.noob.lol@gmail.com',
  url = 'https://github.com/datanooblol/ggwp',
  # check url everytime you release new version
  download_url = 'https://github.com/datanooblol/ggwp/archive/refs/tags/0.0.8.tar.gz',
  keywords = ['ez','rfm','prep', 'data model', 'cohort', 'customer'],
  # dependencies used in your library
  install_requires=[
          'numpy',
          'pandas',
      ],
  classifiers=[  # Optional
    # How mature is this project? Common values are
    #   3 - Alpha
    #   4 - Beta
    #   5 - Production/Stable
    'Development Status :: 4 - Beta',

    # Indicate who your project is intended for
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',

    # Pick your license as you wish
    'License :: OSI Approved :: MIT License',

    # Specify the Python versions you support here. In particular, ensure
    # that you indicate whether you support Python 2, Python 3 or both.
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7'
  ],
)